/***************************************************************************
                          gmpdingo.h  -  My own implementation of the
                                         Go Modem Protocol...
                             -------------------
    begin                : Sat Nov 3 2001
    copyright            : (C) 2001 by Dominique Laigle
    email                : liobadom@internet.lu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GMP_DINGO_H
#define GMP_DINGO_H

#include <stdlib.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/resource.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <signal.h>
#include <pthread.h>
#include <dirent.h>
#include <syslog.h>

#include <glib.h>

#include <gmpmiddleware.h>

#define GMP_PIPE_ERROR  -200
#define GMP_FORK_ERROR  -210

/* Basics commands of the GMP as constants. The fixed bits are part
 * of the send-receive process and not the definitions. When the args are
 * variables, they are AND'ed with the fixed args declared, hence the 111s.
 * Due to big or little endians architectures, they are not as 16 bits
 * values defined, but 2 time 8 bits instead.
 */

#ifndef _GMP_REGISTRATION_
#define _GMP_REGISTRATION_

enum { GMP_CHANNEL_NONE,
       GMP_CH_HUMAN,
       GMP_CH_DEVICE,
       GMP_CH_PROGRAM,
       GMP_CHANNEL_LAST };

typedef struct
      {
        gint   board_size;   /* Right now, only 19x19 is supported */
        gint   rules;        
        gint   handicap;
        gint   time_limit;   /* Main thinking time */
			  gint   byo_stones;   /* Number of stones to play in byo yomi */
			  gint   byo_time;     /* Time limit to play the byo yomi stones */
        gint   black_type;
        gint   white_type;
				gint   black_pid;    /* if channel type is PROGRAM */
				gint   white_pid;    /* if channel type is PROGRAM */
        gchar *black_name;   /* human,device or program name */
        gchar *black_channel;
        gchar *white_name;   /* human,device or program name */
        gchar *white_channel;
      } GMP_REGISTRATION;

#define GMP_HANDSHAKE_OK				10
#define GMP_BS_INVALID				 -10
#define GMP_RU_INVALID         -20
#define GMP_HA_INVALID         -30
#define GMP_TL_INVALID         -40
#define GMP_B_CH_TYPE_INVALID  -50
#define GMP_W_CH_TYPE_INVALID  -60
#define GMP_B_CH_ERROR         -70
#define GMP_W_CH_ERROR         -80
#define GMP_REGISTRY_NOT_EMPTY -90
#define GMP_NOT_REGISTERED     -100

#endif /* _GMP_REGISTRATION_ */

void     gmp_dingo_reset( void );
int      gmp_dingo_init( void );
int      gmp_dingo_clean_registry( void );
int      gmp_dingo_register( GMP_REGISTRATION * );

int      gmp_dingo_get_move( int color, guint *the_move );
int      gmp_dingo_send_move( int color, int the_move );
int      gmp_dingo_undo_move( int color );
int      gmp_dingo_answer_query( int channel, int query_argument );
int      gmp_dingo_start_hanshake( int );

gboolean gmp_channel_is_activated( int );

#endif /* GMP_DINGO_H */
