/***************************************************************************
                          gmplowlevel.h  -  This is the implementation
                                            of the low level of the GMP
                             -------------------
    begin                : Sun Nov 18 2001
    copyright            : (C) 2001 by Dominique Laigle
    email                : liobadom@internet.lu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GMP_LOW_LEVEL_H
#define GMP_LOW_LEVEL_H

#include <stdlib.h>
#include <unistd.h>
#include <time.h> 
#include <sys/time.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/resource.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include <glib.h>

/* All constants defined within the preprocessor instead of the code itself.
 * For a lib, it is important to reduce the heap as much as possible since
 * it is as much that the process this lib is linked against doesn't get...
 */

/* Basics commands of the GMP as constants. The fixed bits are part
 * of the send-receive process and not the definitions. When the args are
 * variables, they are AND'ed with the fixed args declared, hence the 111s.
 */

#define GMP_ACK    0x07
#define GMP_DENY   0x10
#define GMP_RESET  0x20
#define GMP_QUERY  0x30
#define GMP_ANSWER 0x40
#define GMP_MOVE   0x50
#define GMP_UNDO   0x60
#define GMP_XTEND  0x70

/* Queries collection */

#define ASK_GAME			 0
#define ASK_BUFSIZE    1
#define ASK_PROTOCOL   2
#define ASK_STONES     3
#define ASK_BTIME      4
#define ASK_WTIME      5
#define ASK_CHARSET    6
#define ASK_RULES      7
#define ASK_HANDICAP   8
#define ASK_BOARDSIZE  9
#define ASK_TIMELIMIT 10
#define ASK_COLOR     11
#define ASK_WHO       12
#define ASK_EXTEND    13 /* used too as the last bound of queries collection */

/* answer collection  */

#define ANS_UNKOWN      0
#define ANS_GO_GAME     0
#define ANS_BUFF_4	    0
#define ANS_PROTOCOL    0
#define ANS_JAP_RULES   1
#define ANS_ING_RULES   2
#define ANS_BOARD_SIZE 19
#define ANS_WHITE       1
#define ANS_BLACK       2
#define ANS_HANDICAP    1 /* even games */

#define GMP_START_BYTE_MASK 0xfc


/* The most common formats used to internally represent a Go Position.
 * The purpose, as library, is to be able to convert any of those
 * formats to the 1...360 awaited by the GMP.
 * In any format, however, must a pass move be given as a 0 value.
 */

enum { FORMAT_UNKNOWN, FORMAT_XY, FORMAT_421, FORMAT_360, LAST_FORMAT };

#define GMP_FORMAT_ERROR -10
#define GMP_PACKET_ERROR -20
#define GMP_CH_IO_ERROR   -1
#define GMP_PACKET_OK      1

/* For all the functions defined here, the channel 0 is black, 1 is white */

#define GMP_BLACK_CHANNEL 0
#define GMP_WHITE_CHANNEL 1

enum { GMP_WAIT_NOTHING, GMP_WAIT_ANSWER, GMP_WAIT_ACK };

#ifndef _GMP_CMD_FIFO_
#define _GMP_CMD_FIFO_

#define GMP_FIFO_MAX_SIZE 64 /* should be enough */

typedef struct {
                 gint			   fd_out;
                 gint			   fd_in;
                 guchar      packet[4];
                 guchar      received:3; /* 0 to 4 bytes received */
                 guchar      send_sequence:1;
                 guchar      recv_sequence:1;
                 guchar      last_recv_seq:1;
                 guchar      retn_sequence:1;
                 guchar      awaiting;
                 gboolean    valid; /* So far: not used */
               } GMP_QUEUE;

typedef struct {
                 int       command;
                 int       argument;
                 char      cmd_txt[16];
                 char      arg_txt[16];
               } GMP_MSG;

#endif /* _GMP_CMD_FIFO_ */

/* Initializations */
#ifdef _GMP_DEBUG							 
  void gmp_lowlevel_debug( void );
#endif							 
void gmp_lowlevel_initialize( GMP_QUEUE *q_channel, int color );
void gmp_lowlevel_cleanup( GMP_QUEUE *q_channel, int color );
							 
/* protocol functions */

int gmp_lowlevel_read_channel( GMP_QUEUE *q_ch, gboolean time_out );
int gmp_lowlevel_send_message( GMP_QUEUE *q_ch, int col, gboolean inv_seq );
int gmp_lowlevel_send_command( GMP_QUEUE *q_ch, int col, guchar cmd, guchar arg );		 

/* Usefull functions */

void gmp_lowlevel_channel_copy( GMP_QUEUE *channel_dest, GMP_QUEUE *channel_src );
int  gmp_lowlevel_parse_message( guchar *msg, GMP_MSG *parsed );


#endif /* GMP_LOW_LEVEL_H */
