/***************************************************************************
                          guigoban.h  -
                             -------------------
    begin                : Sun Sep 16 2001
    copyright            : (C) 2001 by Dominique Laigle
    email                : liobadom@internet.lu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GUIGOBAN_H
#define GUIGOBAN_H

#include <gtk/gtk.h>
#include <gdk_imlib.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define GUI_GOBAN(obj)		            ( GTK_CHECK_CAST( (obj), gui_goban_get_type(), GuiGoban) )
#define GUI_GOBAN_CLASS(klass)	      ( GTK_CHECK_CLASS_CAST( (klass), GTK_TYPE_GUI_GOBAN, GuiGobanClass) )
#define GTK_IS_GUI_GOBAN(obj)		      ( GTK_CHECK_TYPE( (obj), gui_goban_get_type() ) )
#define GTK_IS_GUI_GOBAN_CLASS(klass) ( GTK_CHECK_CLASS_TYPE( (klass), gui_goban_get_type()) )

#ifndef GOBAN_CONTENT
#define GOBAN_CONTENT

/* goban contents */

enum {
       EMPTY,
       WHITE_STONE,
       BLACK_STONE,
       MARK,
       EDGE,
       DAME_POINT,
       WHITE_POINT,
       BLACK_POINT,
       DEAD_STONE,
       WHITE_DEAD,
       BLACK_DEAD,
       UNKNOWN_VALUE
     };

#endif

/* Somme errors messages... */

enum {
       NO_ERROR,
       POINT_NOT_FREE,
       STRING_SUICIDE,
       KO_VIOLATION
     };

#ifndef GO_REC
#define GO_REC
		 
typedef struct { guchar  start_pos[421]; /* Starting position (handicap or whatever) */
                 gushort move[512];      /* List of played moves */
                 gushort last_move;	     /* Number of played moves */
                 guchar  first_stone;    /* Color of the first played stone */
                 gfloat  komi;           /* 5.5 in japaneese rules, 0.5 in chineese */
               } GO_RECORD;
#endif

#ifndef GO_GUI_MODES
#define GO_GUI_MODES

enum { INIT_STATE, PLAY_STATE, EDIT_STATE, SCORE_STATE, UNKNOW_STATE,
       SILENT_MODE, ORDINAL_MODE, UNKNOW_MODE
     };

#endif

#define RWD -1
#define BST -2
#define SST -3
#define FWD -4
		 
struct _GuiGoban
{
  GtkWidget widget;

  guchar  goban[421]; /* EMPTY, WHITE_STONE or BLACK_STONE */
  guchar  score[421]; /* DAME_POINT, WHITE_POINT, BLACK_POINT or > DEAD_STONE */
  gushort ko;
  gushort whites_removed,
          blacks_removed;

  guchar  active_color,
          gui_actual_mode,
          gui_actual_state,
          last_error,
          nb_pass;

  gboolean mouse_inhibit_flag,
           move_available,
           game_over,
           paint_local;

  GO_RECORD go_game;
};

typedef struct _GuiGoban	      GuiGoban;
typedef struct _GuiGobanClass   GuiGobanClass;

struct _GuiGobanClass
{
  GtkWidgetClass parent_class;

};

/* Methods of the Interface. */

GtkType	   gui_goban_get_type( void );
GtkWidget* gui_goban_new( void );

void     gui_goban_reset( GuiGoban * );

void     gui_goban_set_state( GuiGoban *, guchar );
guchar   gui_goban_get_state( GuiGoban * );

void     gui_goban_set_mode( GuiGoban *, guchar );
guchar   gui_goban_get_mode( GuiGoban * );

void     gui_goban_set_active_color( GuiGoban *, guchar );
guchar   gui_goban_get_active_color( GuiGoban * );

gboolean gui_goban_play_move( GuiGoban *, gushort );
void     gui_goban_replay_to_move( GuiGoban *, gint );
int      gui_goban_get_last_move( GuiGoban * );

void     gui_goban_load_start_position( GuiGoban * );

void     gui_goban_get_prisonners( GuiGoban *, gushort *, gushort * );
void     gui_goban_game_score( GuiGoban *, gfloat *, gfloat * );

void     gui_goban_set_go_game( GuiGoban *, GO_RECORD * );
void     gui_goban_get_go_game( GuiGoban *, GO_RECORD * );
void     gui_goban_inhibit_mouse( GuiGoban *, gboolean );

gint     gui_goban_get_last_error( GuiGoban * );

gboolean gui_goban_game_is_over( GuiGoban * );

/* Interface of the scoring dedicated module gui_score.c */

/* goban scanning vectors */

#define SUD	  -20
#define NORD	+20
#define OUEST	 -1
#define EST	   +1

/* scoringfunctions */

extern void   gui_goban_set_score( GuiGoban *goban_widget, gushort point, int color );
extern void   gui_score_auto_score( GuiGoban * );
extern void   gui_score_propagate_death( GuiGoban *goban_widget, gint point );
extern void   gui_score_propagate_life( GuiGoban *goban_widget, gint point, gint color );
extern gint   gui_score_scan_bloc( guchar *goban_widget, gushort the_point_id, gushort *scanned_str );
extern void   gui_score_guess_score( GuiGoban *goban_widget );
extern void   gui_score_toggle_dead_alive_strings( GuiGoban *goban_widget );

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* GUIGOBAN_H */

