/***************************************************************************
                          guigobanclock.h  -  description
                             -------------------
    begin                : Sat Aou 10 2002
    copyright            : (C) 2002 by Dominique Laigle
    email                : liobadom@internet.lu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GUI_GOBAN_CLOCK_H
#define GUI_GOBAN_CLOCK_H

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/time.h>

#include <gdk/gdk.h>
#include <gtk/gtksignal.h>
#include <gtk/gtkwidget.h>
#include <gtk/gtkmain.h>

/*
 * This widget is an extension of GtkClock that takes in account the
 * canadian byo yomi rules and offer a LCD look & Feel. The byo yomi 
 * settings have the same semantic as defined in the GTP version 2:
 *
 * - main time = 0 : immediate byo yomi,
 * - byoyomi time = 0 : no byo yomi at all
 * - byoyomi time > 0 AND byoyomi stones = 0 : no time limit.   
 */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define GUI_GOBAN_CLOCK(obj)		        ( GTK_CHECK_CAST ((obj), gui_goban_clock_get_type(), GuiGobanClock))
#define GUI_GOBAN_CLOCK_CLASS(klass)	  ( GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_FLAG_BINAIRE, GuiGobanClockClass))
#define GTK_IS_GOBAN_CLOCK(obj)		      ( GTK_CHECK_TYPE ( (obj), gui_goban_clock_get_type() ) )
#define GTK_IS_GOBAN_CLOCK_CLASS(klass) ( GTK_CHECK_CLASS_TYPE ((klass), gui_goban_clock_get_type()))

#define GOBAN_CLOCK_RUN   1
#define GOBAN_CLOCK_STOP  2

typedef struct _GuiGobanClock	      GuiGobanClock;
typedef struct _GuiGobanClockClass  GuiGobanClockClass;

struct _GuiGobanClock
{
  GtkWidget widget;

  GdkColor  back_col,    /* background */
            shadow_col;  /* color of the shadow of the digit */

	gint      time,        /* in seconds */
		        stones,
	          byo_time,    /* in seconds */
		        byo_stones;  /* to play within byo yomi (canadian style) */

	gint      state,       /* RUN or STOP */
		        handler_id;  /* id of the timeout function */
		
	gint      seg_w,
		        seg_h,
		        w_digit,
		        w_digit_beg,
		        h_digit_beg,
		        h_digit_mit,
		        h_digit_end;
		
	gint      base_x[5],
		        base_y;
		
	guchar    digit[5];   /* digits to display HMMSS */
	gboolean  blink,      /* if the display has to blink */
	          flip_flop,  /* TRUE=on, FALSE=off */
						byo_yomi,   /* TRUE if we entered the byo yomi */						
						time_limit; 
};

struct _GuiGobanClockClass
{
  GtkWidgetClass parent_class;			
};

GtkType	   gui_goban_clock_get_type( void );
GtkWidget* gui_goban_clock_new( void );

void gui_goban_clock_reset( GuiGobanClock * );
void gui_goban_clock_set_time( GuiGobanClock *, gint );
void gui_goban_clock_set_byo_yomi( GuiGobanClock *, gint, gint );
void gui_goban_clock_start( GuiGobanClock * );
void gui_goban_clock_play( GuiGobanClock * );
void gui_goban_clock_stop( GuiGobanClock * );

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* GUI_GOBAN_CLOCK_H */
